#!/usr/bin/env python

import os
import sys
sys.path.append("..")
sys.path.append(os.path.join(sys.path[0], '..'))

from twisted.internet import reactor, protocol, defer, task
from twisted.python import util

from gearman import client

@defer.inlineCallbacks
def gclient(gearman):
    w = client.GearmanClient(gearman)

    x = yield w.submit('test', 'some data')
    print "result:", repr(x)
    reactor.stop()

d=protocol.ClientCreator(reactor, client.GearmanProtocol).connectTCP(
    sys.argv[1], 4730)
d.addCallback(gclient)

# sys.settrace(util.spewer)

reactor.run()




#!/usr/bin/env python

import os
import sys
sys.path.append("..")
sys.path.append(os.path.join(sys.path[0], '..'))

from twisted.internet import reactor, protocol, defer, task

from gearman import client

@defer.inlineCallbacks
def worker(gearman):
    rv = yield gearman.echo()
    print "Echo 1 yielded %d: ``%s''" % rv
    rv = yield gearman.echo("something else")
    print "Echo 2 yielded %d: ``%s''" % rv

    d1 = gearman.echo("pipeline 1")
    d1.addCallback(lambda v: sys.stdout.write(
            "Echo 3 yielded %d: ``%s''\n" % v))
    d2 = gearman.echo("pipeline 2")
    d2.addCallback(lambda v: sys.stdout.write(
            "Echo 4 yielded %d: ``%s''\n" % v))

    dl = defer.DeferredList([d1, d2])
    yield dl

    print "OK.  We're done."

    reactor.stop()

d=protocol.ClientCreator(reactor, client.GearmanProtocol).connectTCP(
    sys.argv[1], 4730)
d.addCallback(worker)

reactor.run()




"""
Gearman client implementation.
"""

import sys
import struct

from collections import deque

from twisted.internet import defer
from twisted.protocols import stateful
from twisted.python import log

from constants import *

__all__ = ['GearmanProtocol', 'GearmanWorker', 'GearmanClient']

class GearmanProtocol(stateful.StatefulProtocol):
    """Base protocol for handling gearman connections."""

    unsolicited = [ WORK_COMPLETE, WORK_FAIL, NOOP,
                    WORK_DATA, WORK_WARNING, WORK_EXCEPTION ]

    def connectionMade(self):
        self.receivingCommand = 0
        self.deferreds = deque()
        self.unsolicited_handlers = set()

    def send_raw(self, cmd, data=''):
        """Send a command with the given data with no response."""

        self.transport.writeSequence([REQ_MAGIC,
                                      struct.pack(">II", cmd, len(data)),
                                      data])

    def send(self, cmd, data=''):
        """Send a command and get a deferred waiting for the response."""
        self.send_raw(cmd, data)
        d = defer.Deferred()
        self.deferreds.append(d)
        return d

    def getInitialState(self):
        return self._headerReceived, HEADER_LEN

    def connectionLost(self, reason):
        for d in list(self.deferreds):
            d.errback(reason)
        self.deferreds.clear()

    def _headerReceived(self, header):
        if header[:4] != RES_MAGIC:
            log.msg("Invalid header magic returned, failing.")
            self.transport.loseConnection()
            return
        cmd, size = struct.unpack(">II", header[4:])

        self.receivingCommand = cmd
        return self._completed, size

    def _completed(self, data):
        if self.receivingCommand in self.unsolicited:
            self._unsolicited(self.receivingCommand, data)
        else:
            d = self.deferreds.popleft()
            d.callback((self.receivingCommand, data))
        self.receivingCommand = 0

        return self._headerReceived, HEADER_LEN

    def _unsolicited(self, cmd, data):
        for cb in self.unsolicited_handlers:
            cb(cmd, data)

    def register_unsolicited(self, cb):
        self.unsolicited_handlers.add(cb)

    def unregister_unsolicited(self, cb):
        self.unsolicited_handlers.discard(cb)

    def echo(self, data="hello"):
        """Send an echo request."""

        return self.send(ECHO_REQ, data)

class _GearmanJob(object):
    """A gearman job."""

    def __init__(self, raw_data):
        self.handle, self.function, self.data = raw_data.split("\0", 2)

    def __repr__(self):
        return "<GearmanJob %s func=%s with %d bytes of data>" % (self.handle,
                                                                  self.function,
                                                                  len(self.data))

class GearmanWorker(object):
    """A gearman worker."""

    def __init__(self, protocol):
        self.protocol = protocol
        self.functions = {}
        self.sleeping = None
        self.protocol.register_unsolicited(self._unsolicited)

    def setId(self, client_id):
        """Set the client ID for monitoring and what-not."""
        self.protocol.send_raw(SET_CLIENT_ID, client_id)

    def registerFunction(self, name, func):
        """Register the ability to perform a function."""

        self.functions[name] = func
        self.protocol.send_raw(CAN_DO, name)

    def _send_job_res(self, cmd, job, data=''):
        self.protocol.send_raw(cmd, job.handle + "\0" + data)

    def _sleep(self):
        if not self.sleeping:
            self.sleeping = defer.Deferred()
            self.protocol.send_raw(PRE_SLEEP)
        return self.sleeping

    def _unsolicited(self, cmd, data):
        assert cmd == NOOP
        if self.sleeping:
            self.sleeping.callback(None)
            self.sleeping = None

    @defer.inlineCallbacks
    def getJob(self):
        """Get the next job."""

        # If we're currently sleeping, attach to the existing sleep.
        if self.sleeping:
            yield self._sleep()

        stuff = yield self.protocol.send(GRAB_JOB)
        while stuff[0] == NO_JOB:
            yield self._sleep()
            stuff = yield self.protocol.send(GRAB_JOB)
        defer.returnValue(_GearmanJob(stuff[1]))

    @defer.inlineCallbacks
    def _finishJob(self, job):
        assert job
        f = self.functions[job.function]
        assert f
        try:
            rv = yield f(job)
            if rv is None:
                rv = ""
            self._send_job_res(WORK_COMPLETE, job, rv)
        except:
            etype, emsg, bt = sys.exc_info()
            self._send_job_res(WORK_EXCEPTION, job, "%s(%s)"
                               % (etype.__name__, emsg))
            self._send_job_res(WORK_FAIL, job)

    def doJob(self):
        """Do a single job"""
        return self.getJob().addCallback(self._finishJob)

    def doJobs(self, keepGoing=lambda: True):
        """Do jobs forever (or until the given function returns False)"""
        while keepGoing():
            yield self.doJob()

class _GearmanJobHandle(object):

    def __init__(self, deferred):
        self._deferred = deferred
        self._work_data = []
        self._work_warning = []

    @property
    def work_data(self):
        return ''.join(self._work_data)

    @property
    def work_warning(self):
        return ''.join(self._work_warning)

class GearmanJobFailed(Exception):
    """Exception thrown when a job fails."""
    pass

class GearmanClient(object):
    """A gearman client.

    Submits jobs and stuff."""

    def __init__(self, protocol):
        self.protocol = protocol
        self.protocol.register_unsolicited(self._unsolicited)
        self.jobs = {}

    def _register(self, job_handle, job):
        self.jobs[job_handle] = job

    def _unsolicited(self, cmd, data):
        if cmd in [ WORK_COMPLETE, WORK_FAIL,
                    WORK_DATA, WORK_WARNING ]:
            pos = data.find("\0")
            if pos == -1:
                handle = data
            else:
                handle = data[:pos]
                data = data[pos+1:]

            j = self.jobs[handle]

            if cmd in [ WORK_COMPLETE, WORK_FAIL]:
                self._jobFinished(cmd, j, handle, data)

    def _jobFinished(self, cmd, job, handle, data):
        # Delete the job if it's finished
        del self.jobs[handle]

        if cmd == WORK_COMPLETE:
            job._deferred.callback(data)
        elif cmd == WORK_FAIL:
            job._deferred.errback(GearmanJobFailed())

    def _submit(self, cmd, function, data, unique_id):

        def _submitted(x, d):
            self._register(x[1], _GearmanJobHandle(d))

        d = self.protocol.send(cmd,
                               function + "\0" + unique_id + "\0" + data)

        rv = defer.Deferred()
        d.addCallback(_submitted, rv)

        return rv

    def submit(self, function, data, unique_id=''):
        """Submit a job with the given function name and data."""
        return self._submit(SUBMIT_JOB, function, data, unique_id)

    def submitHigh(self, function, data, unique_id=''):
        """Submit a high priority job with the given function name and data."""
        return self._submit(SUBMIT_JOB_HIGH, function, data, unique_id)

    def submitLow(self, function, data, unique_id=''):
        """Submit a low priority job with the given function name and data."""
        return self._submit(SUBMIT_JOB_LOW, function, data, unique_id)

    def _submitBg(self, cmd, function, data, unique_id):
        return self.protocol.send(cmd,
                                  function + "\0" + unique_id + "\0" + data)

    def submitBackground(self, function, data, unique_id=''):
        """Submit a job for background execution."""
        return self._submitBg(SUBMIT_JOB_BG, function, data, unique_id)

    def submitBackgroundLow(self, function, data, unique_id=''):
        """Submit a job for background execution at low priority."""
        return self._submitBg(SUBMIT_JOB_LOW_BG, function, data, unique_id)

    def submitBackgroundHigh(self, function, data, unique_id=''):
        """Submit a job for background execution at high priority."""
        return self._submitBg(SUBMIT_JOB_HIGH_BG, function, data, unique_id)



#!/usr/bin/env python

import os
import sys
sys.path.append("..")
sys.path.append(os.path.join(sys.path[0], '..'))

from twisted.internet import reactor, protocol, defer, task
from twisted.python import util

from gearman import client

def run_test(j):
    print repr(j)
    return j

# @defer.inlineCallbacks
def worker(gearman):
    w = client.GearmanWorker(gearman)
    w.setId("exampleworker")
    w.registerFunction("test", run_test)

    coop = task.Cooperator()
    for i in range(5):
        ## Ramp up start
        reactor.callLater(0.1 * i, lambda: coop.coiterate(w.doJobs()))
        ## Immediate start
        # coop.coiterate(iter(w))

d=protocol.ClientCreator(reactor, client.GearmanProtocol).connectTCP(
    sys.argv[1], 4730)
d.addCallback(worker)

# sys.settrace(util.spewer)

reactor.run()




from twisted.application import internet, service
from twisted.internet import reactor, protocol, task

import client, cjson, traceback

class GearmanWorkerFactory(protocol.ClientFactory):
    protocol = client.GearmanProtocol

    def __init__(self, concurrency=5, id=None):
        self.concurrency = concurrency
        self.id = id
        self.functions = []

    def clientConnectionLost(self, connector, reason):
        """If we get disconnected, reconnect to server."""
        connector.connect()

    def clientConnectionFailed(self, connector, reason):
        print "connection failed:", reason
        reactor.stop()
    
    def addFunction(self, func, name=None):
        name = name or func.__name__
        
        self.functions.append((name, expose(func)))
    
    def startWork(self, gm):
        w = client.GearmanWorker(gm)
        
        if self.id: w.setId(self.id)
        
        for name, func in self.functions:
            w.registerFunction(name, func)
        
        coop = task.Cooperator()
        
        for i in range(self.concurrency):
            reactor.callLater(0.1 * i, lambda: coop.coiterate(w.doJobs()))

class GearmanWorkerService(service.Service):
    def __init__(self, host, port, factory):
        self.host = host
        self.port = int(port)
        self.factory = factory
    
    def startService(self):
        reactor.callWhenRunning(self.startListening)
    
    def startListening(self):
        c = protocol.ClientCreator(reactor, self.factory.protocol)
        d = c.connectTCP(self.host, self.port)
        d.addCallback(self.factory.startWork)

def expose(f):
    def wrapped(job):
        args = cjson.decode(job)
        
        try:
            d = f(*args)
            d.addCallback(cjson.encode)
            return d
        except Exception, e:
            return cjson.encode({'error': traceback.print_tb(e) + str(e)})
    
    return wrapped


import struct
from collections import deque

from zope.interface import implements

from twisted.trial import unittest
from twisted.internet import interfaces, reactor, defer

import client, constants

class TestTransport(object):

    implements(interfaces.ITransport)

    disconnecting = False

    def __init__(self):
        self.received = []
        self.disconnected = 0

    def write(self, data):
        self.received.append(data)

    def writeSequence(self, data):
        self.received.extend(data)

    def loseConnection(self):
        self.disconnected += 1

    def getPeer(self):
        return None

    def getHost(self):
        return None

class ExpectedFailure(Exception):
    pass

class ProtocolTestCase(unittest.TestCase):

    def setUp(self):
        self.trans = TestTransport()
        self.gp = client.GearmanProtocol()
        self.gp.makeConnection(self.trans)

    def assertReceived(self, cmd, data):
        self.assertEquals(["\0REQ",
                           struct.pack(">II", cmd, len(data)),
                           data],
                          self.trans.received[:3])
        self.trans.received = self.trans.received[3:]

    def write_response(self, cmd, data):
        self.gp.dataReceived("\0RES")
        self.gp.dataReceived(struct.pack(">II", cmd, len(data)))
        self.gp.dataReceived(data)

class GearmanProtocolTest(ProtocolTestCase):

    def test_makeConnection(self):
        self.assertEquals(0, self.gp.receivingCommand)
        self.assertEquals([], list(self.gp.deferreds))
        self.assertEquals([], list(self.gp.unsolicited_handlers))

    def test_send_raw(self):
        self.gp.send_raw(11, "some data")
        self.assertReceived(11, "some data")
        self.assertEquals(0, len(self.gp.deferreds))

    def test_send(self):
        self.gp.send(11, "some data")
        self.assertReceived(11, "some data")
        self.assertEquals(1, len(self.gp.deferreds))

    def test_connectionLost(self):
        d = self.gp.send(11, "test")
        d.addCallback(lambda x: unittest.FailTest())
        d.addErrback(lambda x: x.trap(ExpectedFailure))
        self.gp.connectionLost(ExpectedFailure())
        return d

    def test_badResponse(self):
        self.assertEquals(0, self.trans.disconnected)
        self.trans.shouldLoseConnection = True
        self.gp.dataReceived("X" * constants.HEADER_LEN)
        reactor.callLater(0, self.assertEquals, 1, self.trans.disconnected)

    def test_send_echo(self):
        d = self.gp.echo()
        self.assertReceived(constants.ECHO_REQ, "hello")

    def test_echoRt(self):
        """Test an echo round trip."""
        d = self.gp.echo()
        d.addCallback(lambda x:
                          self.assertEquals(x,
                                            (constants.ECHO_RES, "hello")))
        self.write_response(constants.ECHO_RES, "hello")
        return d

    def test_register_unsolicited(self):
        def cb(cmd, data):
            pass
        self.gp.register_unsolicited(cb)
        self.assertEquals(1, len(self.gp.unsolicited_handlers))
        self.gp.register_unsolicited(cb)
        self.assertEquals(1, len(self.gp.unsolicited_handlers))
        self.gp.register_unsolicited(lambda a,b: True)
        self.assertEquals(2, len(self.gp.unsolicited_handlers))

    def test_unregister_unsolicited(self):
        def cb(cmd, data):
            pass
        self.gp.register_unsolicited(cb)
        self.assertEquals(1, len(self.gp.unsolicited_handlers))
        self.gp.unregister_unsolicited(cb)
        self.assertEquals(0, len(self.gp.unsolicited_handlers))

    def test_unsolicitedCallbackHandling(self):
        d = defer.Deferred()
        self.gp.register_unsolicited(lambda cmd, data: d.callback(True))
        self.write_response(constants.WORK_COMPLETE, "test\0")
        return d

class GearmanJobTest(unittest.TestCase):

    def test_constructor(self):
        gj = client._GearmanJob("footdle\0dys\0some data")
        self.assertEquals("footdle", gj.handle)
        self.assertEquals("dys", gj.function)
        self.assertEquals("some data", gj.data)

        self.assertEquals("<GearmanJob footdle func=dys with 9 bytes of data>",
                          repr(gj))

class GearmanWorkerTest(ProtocolTestCase):

    def setUp(self):
        super(GearmanWorkerTest, self).setUp()
        self.gw = client.GearmanWorker(self.gp)

    def test_registerFunction(self):
        self.gw.registerFunction("awesomeness", lambda x: True)
        self.assertReceived(constants.CAN_DO, "awesomeness")

    def test_sendingJobResponse(self):
        job = client._GearmanJob("test\0blah\0junk")
        self.gw._send_job_res(constants.WORK_COMPLETE, job, "the value")
        self.assertReceived(constants.WORK_COMPLETE, "test\0the value")

    def test_sleep(self):
        a = []
        for i in range(5):
            a.append(self.gw._sleep())

        self.write_response(constants.NOOP, "")
        return defer.DeferredList(a)

    def test_getJob(self):
        d = self.gw.getJob()
        self.write_response(constants.JOB_ASSIGN,
                            "footdle\0funk\0args and stuff")
        def _handleJob(j):
            self.assertEquals("footdle", j.handle)
            self.assertEquals("funk", j.function)
            self.assertEquals("args and stuff", j.data)

        d.addCallback(_handleJob)
        return d

    def test_getJobWithWaiting(self):
        d = self.gw.getJob()
        self.write_response(constants.NO_JOB, "")
        self.write_response(constants.NOOP, "")
        self.write_response(constants.JOB_ASSIGN,
                            "footdle\0funk\0args and stuff")
        def _handleJob(j):
            self.assertEquals("footdle", j.handle)
            self.assertEquals("funk", j.function)
            self.assertEquals("args and stuff", j.data)

        d.addCallback(_handleJob)
        return d

    def test_getJobWithWaitingMultiNOOP(self):
        d = self.gw.getJob()
        self.write_response(constants.NO_JOB, "")
        self.write_response(constants.NOOP, "")
        self.write_response(constants.NOOP, "")
        self.write_response(constants.NOOP, "")
        self.write_response(constants.JOB_ASSIGN,
                            "footdle\0funk\0args and stuff")
        def _handleJob(j):
            self.assertEquals("footdle", j.handle)
            self.assertEquals("funk", j.function)
            self.assertEquals("args and stuff", j.data)

        d.addCallback(_handleJob)
        return d

    def test_getJobWhileAlreadyWaiting(self):
        sd = self.gw._sleep()
        d = self.gw.getJob()
        self.write_response(constants.NOOP, "")
        self.write_response(constants.JOB_ASSIGN,
                            "footdle\0funk\0args and stuff")
        def _handleJob(j):
            self.assertEquals("footdle", j.handle)
            self.assertEquals("funk", j.function)
            self.assertEquals("args and stuff", j.data)

        d.addCallback(_handleJob)
        return defer.DeferredList([sd, d])

    def test_finishJob(self):
        self.gw.functions['blah'] = lambda x: x.upper()
        job = client._GearmanJob("test\0blah\0junk")
        d = self.gw._finishJob(job)

        d.addCallback(lambda x:
                          self.assertReceived(constants.WORK_COMPLETE,
                                              "test\0JUNK"))

    def test_finishJobNull(self):
        self.gw.functions['blah'] = lambda x: None
        job = client._GearmanJob("test\0blah\0junk")
        d = self.gw._finishJob(job)

        d.addCallback(lambda x:
                          self.assertReceived(constants.WORK_COMPLETE,
                                              "test\0"))

    def test_finishJobException(self):
        def _failing(x):
            raise Exception("failed")
        self.gw.functions['blah'] = _failing
        job = client._GearmanJob("test\0blah\0junk")
        d = self.gw._finishJob(job)

        def _checkReceived(x):
            self.assertReceived(constants.WORK_EXCEPTION,
                                "test\0" + 'Exception(failed)')
            self.assertReceived(constants.WORK_FAIL, "test\0")

        d.addCallback(_checkReceived)

    def test_doJob(self):
        self.gw.functions['blah'] = lambda x: x.upper()
        d = self.gw.doJob()
        self.write_response(constants.JOB_ASSIGN,
                            "footdle\0blah\0args and stuff")

        def _verify(x):
            self.assertReceived(constants.GRAB_JOB, "")
            self.assertReceived(constants.WORK_COMPLETE,
                                "footdle\0ARGS AND STUFF")

        d.addCallback(_verify)
        return d

    def test_doJobs(self):
        self.gw.functions['blah'] = lambda x: x.upper()
        d = self.gw.doJobs().next()
        self.write_response(constants.JOB_ASSIGN,
                            "footdle\0blah\0args and stuff")

        def _verify(x):
            self.assertReceived(constants.GRAB_JOB, "")
            self.assertReceived(constants.WORK_COMPLETE,
                                "footdle\0ARGS AND STUFF")

        d.addCallback(_verify)
        return d

    def test_doJobsNoLoop(self):
        try:
            d = self.gw.doJobs(lambda: False).next()
        except StopIteration:
            pass

    def test_setId(self):
        self.gw.setId("my id")
        self.assertReceived(constants.SET_CLIENT_ID, "my id")

class GearmanJobHandleTest(unittest.TestCase):

    def test_workData(self):
        gjh = client._GearmanJobHandle(None)
        gjh._work_data.extend(['test', 'ing'])
        self.assertEquals('testing', gjh.work_data)

    def test_workWarning(self):
        gjh = client._GearmanJobHandle(None)
        gjh._work_warning.extend(['test', 'ing'])
        self.assertEquals('testing', gjh.work_warning)

class GearmanClientTest(ProtocolTestCase):

    def setUp(self):
        super(GearmanClientTest, self).setUp()
        self.gc = client.GearmanClient(self.gp)

    def test_unsolicitedUnused(self):
        self.gc._register('x', client._GearmanJobHandle(None))
        self.gc._unsolicited(constants.WORK_DATA, "x\0some data")

    def test_unsolicitedUnusedNoData(self):
        self.gc._register('x', client._GearmanJobHandle(None))
        self.gc._unsolicited(constants.WORK_DATA, "x")

    def test_finishJob(self):
        d = defer.Deferred()
        self.gc._register('x', client._GearmanJobHandle(d))
        self.gc._unsolicited(constants.WORK_COMPLETE, "x\0some data")

        d.addCallback(lambda x: self.assertEquals("some data", x))
        return d

    def test_failJob(self):
        d = defer.Deferred()
        self.gc._register('x', client._GearmanJobHandle(d))
        self.gc._unsolicited(constants.WORK_FAIL, "x\0some data")

        d.addErrback(lambda x: x.trap(client.GearmanJobFailed))
        return d

    def test_submit(self):
        d = self.gc.submit('test', 'test data')
        self.assertReceived(constants.SUBMIT_JOB, 'test\0\0test data')
        self.write_response(constants.JOB_CREATED, 'test_submit')
        self.write_response(constants.WORK_COMPLETE,
                            'test_submit\0done')
        d.addCallback(lambda x: self.assertEquals("done", x))
        return d

    def test_submitHigh(self):
        d = self.gc.submitHigh('test', 'test data')
        self.assertReceived(constants.SUBMIT_JOB_HIGH, 'test\0\0test data')
        self.write_response(constants.JOB_CREATED, 'test_submit')
        self.write_response(constants.WORK_COMPLETE,
                            'test_submit\0done')
        d.addCallback(lambda x: self.assertEquals("done", x))
        return d

    def test_submitLow(self):
        d = self.gc.submitLow('test', 'test data', 'uniqid')
        self.assertReceived(constants.SUBMIT_JOB_LOW, 'test\0uniqid\0test data')
        self.write_response(constants.JOB_CREATED, 'test_submit')
        self.write_response(constants.WORK_COMPLETE,
                            'test_submit\0done')
        d.addCallback(lambda x: self.assertEquals("done", x))
        return d

    def test_submitBackground(self):
        d = self.gc.submitBackground('test', 'test data')
        self.assertReceived(constants.SUBMIT_JOB_BG, 'test\0\0test data')
        self.write_response(constants.JOB_CREATED, 'test_submit')
        return d

    def test_submitBackgroundLow(self):
        d = self.gc.submitBackgroundLow('test', 'test data')
        self.assertReceived(constants.SUBMIT_JOB_LOW_BG, 'test\0\0test data')
        self.write_response(constants.JOB_CREATED, 'test_submit')
        return d

    def test_submitBackgroundHigh(self):
        d = self.gc.submitBackgroundHigh('test', 'test data')
        self.assertReceived(constants.SUBMIT_JOB_HIGH_BG, 'test\0\0test data')
        self.write_response(constants.JOB_CREATED, 'test_submit')
        return d



"""Protocol definitions."""

import struct

REQ_MAGIC          = "\0REQ"
RES_MAGIC          = "\0RES"

CAN_DO             = 1
CANT_DO            = 2
RESET_ABILITIES    = 3
PRE_SLEEP          = 4
NOOP               = 6
SUBMIT_JOB         = 7
JOB_CREATED        = 8
GRAB_JOB           = 9
NO_JOB             = 10
JOB_ASSIGN         = 11
WORK_STATUS        = 12
WORK_COMPLETE      = 13
WORK_FAIL          = 14
GET_STATUS         = 15
ECHO_REQ           = 16
ECHO_RES           = 17
SUBMIT_JOB_BG      = 18
ERROR              = 19
STATUS_RES         = 20
SUBMIT_JOB_HIGH    = 21
SET_CLIENT_ID      = 22
CAN_DO_TIMEOUT     = 23
ALL_YOURS          = 24
WORK_EXCEPTION     = 25
OPTION_REQ         = 26
OPTION_RES         = 27
WORK_DATA          = 28
WORK_WARNING       = 29
GRAB_JOB_UNIQ      = 30
JOB_ASSIGN_UNIQ    = 31
SUBMIT_JOB_HIGH_BG = 32
SUBMIT_JOB_LOW     = 33
SUBMIT_JOB_LOW_BG  = 34
SUBMIT_JOB_SCHED   = 35
SUBMIT_JOB_EPOCH   = 36

# Magic, type, data size
PKT_FMT = ">III"
HEADER_LEN = struct.calcsize(PKT_FMT)



#!/usr/bin/env python
"""
TwistedGears installation script
"""
from setuptools import setup

setup(
    name = "twisted-gears",
    version = "0.2",
    author = "Dustin Sallings",
    author_email = "dustin@spy.net",
    url = "http://bleu.west.spy.net/~dustin/",
    description = "A twisted interface to gearman.",
    license="Unknown",
    packages = ["gearman"],
    classifiers = [
        "Framework :: Twisted",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Topic :: Communications",
        "Topic :: Software Development :: Libraries :: Python Modules"
        ]
    )
